  MEMBER

  PRAGMA('project(#compile libcurl.clw)')
  PRAGMA('project(#pragma link(libcurl.lib))')
  PRAGMA('link(YKassa.ico)')

  Include('YKassa.inc'),Once

  Map
    Module('WinAPI')
      !   GUID
      YKassa::CoCreateGuid(*?),UNSIGNED,PASCAL,Raw,Name('CoCreateGuid')
      YKassa::StringFromGUID2(*?,*?,LONG cchMax),UNSIGNED,PASCAL,Raw,Name('StringFromGUID2')
      YKassa::WideCharToMultiByte(UNSIGNED Codepage=0,USHORT dwFlags=0,ULong LpWideCharStr, Long cchWideChar, ULong lpMultuByteStr, |
                                  Long cbMultiByte,Ulong LpDefalutChar,BYTE lpUsedDefalutChar),Raw,UNSIGNED,Pascal,Name('WideCharToMultiByte')

      YKassa::FnSplit(*cstring,*cstring,*cstring,*cstring,*cstring),short,Raw,Proc,Name('_fnsplit')
      YKassa::ShellExecute(LONG phWnd, <*CSTRING pAction>, *CSTRING pFilename, <*CSTRING pParameters>, <*CSTRING pDirectory>, |
                           LONG pFlags=1), LONG, PASCAL, RAW, PROC, NAME('ShellExecuteA')
    end
  
    Module('YKassa.clw')                                      !  
      YKassa::ExtFileName(String xIPath, Byte xVid),String    !          
      YKassa::EscapeString(String sValue)String               !  JSON 
    end
    
    !    : https://github.com/mikeduglas/printf
    Include('printf.inc'),Once ! -> printf('%v',) -> BASE64
  end

YKassa.Construct  Procedure
  Code
  Self.Curl &= New TCurlClass
  Self.Curl.Init()

  Self.ResponseBuffer &= NewDynStr()          !   
  
YKassa.Destruct   Procedure
  Code
  Dispose(Self.Curl)

  If Not (Self.ResponseBuffer &= Null)        !   
     Self.ResponseBuffer.Kill
     DisposeDynStr(Self.ResponseBuffer)
     Self.ResponseBuffer &= Null
  end
  
  Self.FlagInit = False                       !   

YKassa.Init       Procedure(String sShopID,String sSecretKey,Long lDebugCodePage = -1) !,Bool,Proc
  Code
  Self.FlagInit = False                       !    ()

  If Clip(Left(sShopID)) <> '' and Clip(Left(sSecretKey)) <> ''
     Self.ShopID     = Clip(Left(sShopID))     ! ID 
     Self.SecretKey  = Clip(Left(sSecretKey))  !  
  else
     YKassa::Message(' ID        Kassa !')
  end
                                       !   
  Self.TestShop      = Choose(Sub(Clip(Left(sSecretKey)),1,4) = 'test') 
     
  Self.FlagInit      = True            !   
  Self.DebugCodePage = lDebugCodePage  !       (  = -1 / CP_UTF8 / CP_ACP)
  Self.SetTimeOut()                    !        Curl  

  Return Self.FlagInit
  
!!!    
YKassa.ShowInfoText  Procedure(<String sText>)
  Code
  !      
  If Self.InfoText > 0
     If Not Omitted(sText)
        Self.InfoText{Prop:Text} = Clip(sText)
     else
        Self.InfoText{Prop:Text} = ''
     end
     Display(Self.InfoText)
  end    
 
!!!        Curl 
YKassa.SetTimeOut Procedure(Long inTimeOut = 5)
  Code
  Self.Curl.SetOpt(CURLOPT_CONNECTTIMEOUT, inTimeOut)
  
!!!   Curl
YKassa.SetHeaderCurl  Procedure(String sTypeRequest,<String sIdKey>)  !,Bool,Proc
lResult  CurlCode,Auto      !      
  Code
  Self.ResponseBuffer.Kill  !     
  
  Self.Curl.FreeHttpHeaders()
  If Not Omitted(sIdKey) then Self.Curl.AddHttpHeader('Idempotence-Key: ' & Clip(Left(sIdKey))).
  
  Self.Curl.AddHttpHeader('Content-Type: application/json')  
  Self.Curl.SetHttpHeaders()
  
 !Self.Curl.SetSSLVerifyHost(False)
  Self.Curl.SetSSLVerifyPeer(False)
 !Self.Curl.SetSSLVersion(CURL_SSLVERSION_DEFAULT)    

  lResult = Self.Curl.SetUserPwd(Self.ShopID,Self.SecretKey)  
  lResult = Self.Curl.SetCustomRequest(sTypeRequest)
  
  If lResult = CURLE_OK and sTypeRequest = 'GET' then Self.Curl.SetHttpGET(True).
  
  Return Choose(lResult = CURLE_OK)
  
!!!   
YKassa.CheckResponse  Procedure(*CurlCode xResult_,String xPlace_,Long lExceptionCode = -137137) !,Bool
Json        cJSONFactory
gResponse   Group
type          CString( 41)
id            CString( 37)
code          CString( 81)
description   CString(512)
parameter     CString(256)
            end
  Code
  If Not xResult_ = CURLE_OK
     YKassa::Message(' Curl: ' & xPlace_ & '|' & Self.Curl.StrError(xResult_))

  elsIf InList(Self.Curl.GetResponseCode(),200,lExceptionCode) = 0
  
     If Json.ToGroup(Self.ResponseBuffer,gResponse)
        YKassa::Message('  : ' & xPlace_                & '|'  & |
                        ' : ' & Self.Curl.GetResponseCode() & '||' & |
                        ' : ' & gResponse.Code              & '||' & |
                        gResponse.description                        & '||' & |
                        gResponse.parameter)
     else
        YKassa::Message(' : '    & xPlace_           & '||' & | 
                        ' : ' & Self.Curl.GetResponseCode() & '|'  & | 
                        json::FromUTF8(Self.ResponseBuffer.Str()))
     end

     xResult_ = 99
  end

  Return Choose(xResult_ = CURLE_OK)

!!!       
YKassa.Debug   Procedure(String sFileName, String sText,     Long lTextCodePage = CP_UTF8)
  Code
  If    Self.DebugCodePage = -1  !   
  elsIf Self.DebugCodePage = lTextCodePage then json::SaveFile(YKassa:MyDebugPath & sFileName & '.json', sText)
  else  json::SaveFile(YKassa:MyDebugPath & sFileName & '.json', json::ConvertEncoding(sText, lTextCodePage, Self.DebugCodePage)).
  
!!!       
YKassa.Debug   Procedure(String sFileName, *IDynStr iBuffer, Long lTextCodePage = CP_UTF8)
  Code
  If Not iBuffer &= Null then Self.Debug(sFileName,iBuffer.Str(),lTextCodePage).

!!!   JSON     
YKassa.Debug   Procedure(String sFileName, *cJSON jObject,   Long lTextCodePage = CP_UTF8)
  Code
  If Not jObject &= Null then Self.Debug(sFileName,jObject.ToString(YKassa:FormatJson),lTextCodePage).
  
!!!    / 
YKassa.AskMe   Procedure(*YKassa:G_AskMe gAskMe,Bool bDisplay = False) !,Bool,Proc  
lResult    CurlCode,Auto
Json       cJSONFactory
bRetValue  Bool(False)
bCount     Byte,Auto
sPM        CString(1024)  !   
sPR        CString(41)    !  -
sTMP       CString(41)    !  

  Code
  Self.ShowInfoText('  ')
  
  Self.SetHeaderCurl('GET')  
  lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'me','',Self.ResponseBuffer)   
  Self.Debug('AskMe_Response',Self.ResponseBuffer)

  If Self.CheckResponse(lResult,'AskMe')
     bRetValue = Json.ToGroup(Self.ResponseBuffer,'',gAskMe,,'[' & YKassa:J_ALL & ',' & YKassa:J_AskMe & ']')
     
     If bDisplay
        If Not bRetValue then YKassa::Message('       !')
        else        
           Case gAskMe.fiscalization.provider
           Of ''
              sPR = ' '
           Of 'avanpost'
              sPR = 'Kassa 54-'
           Of 'a_qsi'
              sPR = 'aQsi online'
           Of 'atol'
              sPR = ' '
           Of 'business_ru'
              sPR = '.'
           Of 'digital_kassa'
              sPR = 'digitalkassa'
           Of 'evotor'
              sPR = ''
           Of 'first_ofd'
              sPR = ' '
           Of 'kit_invest'
              sPR = Choose(Not gAskMe.test,' ','Kassa 54-')
           Of 'komtet'
              sPR = ' '
           Of 'life_pay'
              sPR = 'LIFE PAY'
           Of 'mertrade'
              sPR = 'Mertrade'
           Of 'modul_kassa'
              sPR = ''
           Of 'rocket'
              sPR = 'RocketR'
           Of 'shtrih_m'
              sPR = 'Orange Data'
           else
              sPR = gAskMe.fiscalization.provider
           end   
        
           Loop bCount = 1 to Maximum(gAskMe.payment_methods,1)
             If gAskMe.payment_methods[bCount] = '' then Break
             else   
                Case gAskMe.payment_methods[bCount]
                Of 'bank_card'
                   sTMP = ' '
                Of 'yoo_money'
                   sTMP = 'Money'
                Of 'sberbank'
                   sTMP = 'SberPay'                
                Of 'tinkoff_bank'
                   sTMP = 'T-Pay'
                Of 'sbp'
                   sTMP = ''
                Of 'sber_loan'
                   sTMP = '    '
                Of 'b2b_sberbank'
                   sTMP = '  '
                Of 'mobile_balance'
                   sTMP = ' '
                Of 'cash'
                   sTMP = ''
                Of 'electronic_certificate'
                   sTMP = ' '                   
                else
                   sTMP = gAskMe.payment_methods[bCount]
                end
             
                sPM = Clip(sPM) & Choose(bCount = 1,'','|' & All(' ',29)) & sTMP                
             end
           end
           
           YKassa::Message('  ' & Format(ToDay(),@D06.) & ' . (' & Format(Clock(),@T04) & ')' & | 
                          '|---------------------------------------------------' & |
                          '|  IdShop:       ' & gAskMe.account_id   & |
                          '| :               ' & gAskMe.itn          & |
                          '|---------------------------------------------------' & |
                          '|:                      ' & Choose(gAskMe.status = 'enabled','','') & | 
                          '|:                       ' & Choose(Not gAskMe.test,          '','') & |
                          '|---------------------------------------------------' & |
                          '|  :   ' & Choose(Not gAskMe.fiscalization.enabled,'','')    & |
                          '| -:   ' & sPR                 & |
                          '|---------------------------------------------------' & |
                          '|  :    ' & sPM                 & |
                          '|---------------------------------------------------' & |
                          '',MSGMODE:FIXEDFONT)
        end
     end   
  end

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue  
  
!!!  
YKassa.PaymentCreate Procedure(*YKassa:G_ReqPayment gReqPayment,*YKassa:G_Payment gPayment,*YKassa:G_Data gData) !,Bool
lResult    CurlCode
Root       &cJson
Json       cJSONFactory
bRetValue  Bool(False)
sInstance  CString(41)

  Code
  Self.ShowInfoText(' ')
  
  If Not gReqPayment.receipt.items &= Null  !     
     sInstance = ',{{"name":"items","instance":' & Instance(gReqPayment.receipt.items,Thread()) & '}'
  end
  
  Root &= Json::CreateObject(gReqPayment,,'[' & YKassa:J_ALL & ',' & YKassa:J_ReqPayment & sInstance & ']')
  If Not Root &= Null     
     Self.Debug('PaymentCreate_Request',Root,CP_ACP)

     !     ->    JSON  ( UTF-8)
     If Clip(Self.UploadToFile) <> '' then json::SaveFile(Self.UploadToFile,Root.ToUtf8(YKassa:FormatJson)).

     !  
     Self.SetHeaderCurl('POST',gData.Idempotence)
     lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'payments',Root.ToUtf8(),Self.ResponseBuffer)
     Self.Debug('PaymentCreate_Response',Self.ResponseBuffer)
     
     Root.Delete()     
     If Self.CheckResponse(lResult,'PaymentCreate')
        If Json.ToGroup(Self.ResponseBuffer,'',gPayment,,'[' & YKassa:J_ALL & ',' & YKassa:J_Payment & ']')
           
           gData.ID       = gPayment.id         ! ID     
           gData.Status   = gPayment.status     !             
           gData.Amount   = gPayment.Amount     !   
           gData.Created  = gPayment.created_at !  /           
           bRetValue      = True                                          
        end   
     end
  end  

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue

!!!    
YKassa.RefundCreate  Procedure(*YKassa:G_ReqRefund gReqRefund,*YKassa:G_Refund gRefund,*YKassa:G_Data gData) !,Bool
lResult    CurlCode
Root       &cJson
Json       cJSONFactory
bRetValue  Bool(False)

  Code
  Self.ShowInfoText(' ')
  
  Root &= Json::CreateObject(gReqRefund,,'[' & YKassa:J_ALL & ']')
  If Not Root &= Null     
     Self.Debug('RefundCreate_Request',Root,CP_ACP)
     
     !     ->    JSON  ( UTF-8)
     If Clip(Self.UploadToFile) <> '' then json::SaveFile(Self.UploadToFile,Root.ToUtf8(YKassa:FormatJson)).

     !  
     Self.SetHeaderCurl('POST',gData.Idempotence)
     lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'refunds',Root.ToUtf8(),Self.ResponseBuffer)
     Self.Debug('RefundCreate_Response',Self.ResponseBuffer)
     
     Root.Delete()     
     If Self.CheckResponse(lResult,'RefundCreate')
        If Json.ToGroup(Self.ResponseBuffer,'',gRefund,,'[' & YKassa:J_ALL & ']')           
        
           gData.ID       = gRefund.id          ! ID 
           gData.Status   = gRefund.status      !  
           gData.Amount   = gRefund.Amount      !  
           gData.Created  = gRefund.created_at  !  /            
           bRetValue      = True
        end
     end
  end 

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue

!!!     
YKassa.PaymentInfo  Procedure(*YKassa:G_Payment gPayment,*YKassa:G_Data gData) !,Bool
lResult    CurlCode
Json       cJSONFactory
bRetValue  Bool(False)

  Code
  Self.ShowInfoText('  ')
  
  Self.SetHeaderCurl('GET')
  lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'payments/' & gData.ID,'',Self.ResponseBuffer)   
  Self.Debug('PaymentInfo_Response',Self.ResponseBuffer)

  If Self.CheckResponse(lResult,'PaymentInfo')
     If Json.ToGroup(Self.ResponseBuffer,'',gPayment,,'[' & YKassa:J_ALL & ',' & YKassa:J_Payment & ']')        
        gData.Status = gPayment.status  !  
        bRetValue    = True
     end
  end

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue
  
!!!   
YKassa.RefundInfo   Procedure(*YKassa:G_Refund  gRefund,*YKassa:G_Data gData) !,Bool  
lResult    CurlCode
Json       cJSONFactory
bRetValue  Bool(False)

  Code
  Self.ShowInfoText('  ')
  
  Self.SetHeaderCurl('GET')
  lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'refunds/' & gData.ID,'',Self.ResponseBuffer)   
  Self.Debug('RefundInfo_Response',Self.ResponseBuffer)

  If Self.CheckResponse(lResult,'RefundInfo')
     If Json.ToGroup(Self.ResponseBuffer,'',gRefund,,'[' & YKassa:J_ALL & ']')
        gData.Status = gRefund.status  !  
        bRetValue    = True
     end
  end

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue
  
!!!   
YKassa.ReceiptInfo  Procedure(*YKassa:G_Data gData) !,Bool
lResult    CurlCode
Json       cJSONFactory
bRetValue  Bool(False)

gResponse  Group
type         CString(21)
items        Group(YKassa:G_Receipt),Dim(2).
           end 
  Code
  Self.ShowInfoText('  ')
  Self.SetHeaderCurl('GET')
  
  lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'receipts?payment_id=' & gData.ID,,Self.ResponseBuffer)   
  Self.Debug('ReceiptInfo_Response',Self.ResponseBuffer)

  If Self.CheckResponse(lResult,'ReceiptInfo')
     If Json.ToGroup(Self.ResponseBuffer,'',gResponse,,'[' & YKassa:J_ALL & ']')
     
        If gResponse.items[1].id <> ''

           If gResponse.items[1].fiscal_provider_id <> ''  !   
              gData.fiscal_document_number = gResponse.items[1].fiscal_document_number  !   ( )  
              gData.tax_system_code        = gResponse.items[1].tax_system_code         ! 1055   
           end
           
           bRetValue    = True
        end   
     end
  end

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue

!!!   (    )
YKassa.PaymentCancel  Procedure(*YKassa:G_Payment gPayment,*YKassa:G_Data gData) !,Bool
lResult    CurlCode
Json       cJSONFactory
bRetValue  Bool(False)

  Code
  Self.ShowInfoText(' ')
  Self.SetHeaderCurl('POST',gData.Idempotence)
  
  lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'payments/' & gPayment.id & '/cancel','',Self.ResponseBuffer)   
  Self.Debug('PaymentCancel_Response',Self.ResponseBuffer)
  
  If Self.CheckResponse(lResult,'PaymentCancel')
     If Json.ToGroup(Self.ResponseBuffer,'',gPayment,,'[' & YKassa:J_ALL & ',' & YKassa:J_Payment & ']')        
        gData.Status = gPayment.status
        bRetValue    = True
     end   
  end

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue
  
!!!   
YKassa.InvoiceCreate  Procedure(*YKassa:G_ReqInvoice gReqInvoice, |
                                *YKassa:G_Invoice gInvoice,       |
                                *YKassa:G_Data gData) !,Bool,Proc  
lResult    CurlCode
Root       &cJson
Json       cJSONFactory
bRetValue  Bool(False)

  Code
  Self.ShowInfoText(' ר')  
  Self.SetHeaderCurl('POST',gData.Idempotence)
  
  Root &= Json::CreateObject(gReqInvoice,,'[' & YKassa:J_ALL & ',' & YKassa:J_ReqInvoice & | 
                                          ',{{"name":"cart","instance":' & Instance(gReqInvoice.cart,Thread()) & '}]')
                
  If Not Root &= Null     
     Self.Debug('InvoiceCreate_Request',Root,CP_ACP)
     lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'invoices',Root.ToUtf8(),Self.ResponseBuffer)
     
     Self.Debug('InvoiceCreate_Response',Self.ResponseBuffer)
     Root.Delete()
     
     If Self.CheckResponse(lResult,'InvoiceCreate')
        If Json.ToGroup(Self.ResponseBuffer,'',gInvoice,,'[' & YKassa:J_ALL & ']')
        
           gData.ID       = gInvoice.id         ! ID     
           gData.Status   = gInvoice.status     !  
           gData.Created  = gInvoice.created_at !  /             
           bRetValue      = True                                          
        end   
     end
  end  

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue
  
!!!   
YKassa.InvoiceInfo  Procedure(*YKassa:G_Invoice gInvoice,*YKassa:G_Data gData) !,Bool
lResult    CurlCode
Json       cJSONFactory
bRetValue  Bool(False)

  Code
  Self.ShowInfoText('  ר')
  
  Self.SetHeaderCurl('GET')
  lResult = Self.Curl.SendRequest(YKassa:EndPointAPI & 'invoices/' & gData.ID,'',Self.ResponseBuffer)   
  Self.Debug('InvoiceInfo_Response',Self.ResponseBuffer)

  If Self.CheckResponse(lResult,'InvoiceInfo')
     If Json.ToGroup(Self.ResponseBuffer,'',gInvoice,,'[' & YKassa:J_ALL & ']')        
        gData.Status = gInvoice.status  !  
       !gData.Status = 'succeeded'        
        bRetValue    = True
     end
  end

  Self.ShowInfoText()
  Self.ResponseBuffer.Kill
  Return bRetValue
  
! --------------------------------------------------------------------------------------------------
! ---   --------------------------------------------------------------------------
! --------------------------------------------------------------------------------------------------

!!!   
YKassa::Message    Procedure(String sText,Byte bAddMode = 0)
  Code
  Message(sText,' API Kassa','~YKassa.ico','&1. ',,MSGMODE:CANCOPY + bAddMode)

!!!   
YKassa::AskMe      Procedure(String sText) !,Bool
  Code
  Return Choose(Message(sText,' API Kassa','~YKassa.ico',Button:Yes + Button:No,Button:Yes) = 2, True, False)
  
!!!     
YKassa::View       Procedure(*Queue qList)
Root    &cJSON
  Code       
  Root &= json::CreateArray(qList)
  If Not Root &= Null
     YKassa::Message(Root.ToString(True))
     Root.Delete()  
  end

!!!  GUID-36
YKassa::GetGUID    Procedure !,String
pGuid      String(16)
buf        String(128)
buf2       String(64)
cbSize     Long
xRetValue  String(36)
  Code
  If Not YKassa::CoCreateGuid(pGUID)
     If YKassa::StringFromGuid2(pGuid, buf, SIZE(buf)/2)
        cbSize = YKassa::WideCharToMultiByte(, , Address(buf), -1, Address(buf2), SIZE(buf2), 0, 0)
        If cbSize > 0 then xRetValue = buf2[2 : 37].
     end
  end
  Return xRetValue

!!!  WEB 
YKassa::OpenWebPage  Procedure(String xLink)
loc_W                &Window
loc_lpOperation      CString('open')
loc_lpFileName       CString(Len(Clip(xLink)) + 1),Auto
loc_lpParameters     CString(255)  ! 
loc_lpDirectory      CString(File:MaxFileName)
uRetHandle           uShort,Auto

  Code
  loc_lpFileName    = Clip(xLink)
 !loc_lpParameters  = ! 
  loc_W            &= Null
  loc_lpDirectory   = System{Prop:DataPath}
  
  uRetHandle        = YKassa::ShellExecute(loc_W{Prop:Handle},loc_lpOperation,loc_lpFileName,loc_lpParameters,loc_lpDirectory,SW_SHOWNORMAL)
  If uRetHandle <= 32 then YKassa::Message(' : ' & loc_lpFileName).

!!!    
YKassa::ExtFileName  Procedure(String xIPath, Byte xVid) !,String
Loc:Path   CString(File:MaxFilePath),Auto
Loc:Dir    CString(File:MaxFilePath),Auto
Loc:Drive  CString(File:MaxFilePath),Auto
Loc:File   CString(File:MaxFilePath),Auto
Loc:Ext    CString(File:MaxFilePath),Auto
  Code
  Loc:Path = Clip(Left(xIPath))
  YKassa::FnSplit(Loc:Path,Loc:Drive,Loc:Dir,Loc:File,Loc:Ext)
  Return Clip(Choose(xVid, Loc:Drive, Loc:Dir, Loc:File, Loc:Ext))  

!!!   /  ( 2005-08-15T15:52:01Z )
YKassa::FormatDate   Procedure(<Long inDate>, <Long inTime>) !, String
  Code
  If Omitted(inDate) then inDate = ToDay().
  If Omitted(inTime) then inTime = Clock().
  Return Format(inDate,@D10-) & 'T' & Format(inTime,@T04) & 'Z'  ! !2005-08-15T15:52:01Z
  
!!!   JSON     / 
YKassa::SetFileISO  Procedure !,String  
  Code
  Return Format(ToDay(),@D010-) & 'T' & Format(Clock(),@T04-) & 'Z' & '.json'  
  
!!!  JSON 
YKassa::EscapeString  Procedure(String sValue) !, String
jTag      &cJson
RetVal    Any
  Code
  jTag  &= json::CreateString(sValue)
  RetVal = jTag.ToString()
  jTag.Delete()
  Return RetVal 
  
!!!    
YKassa::StatusRus  Procedure(Byte bProcessing,*YKassa:G_Data gData) !,String
  Code  
  If InRange(bProcessing,1,2)  
     If    gData.Status = 'pending'             then Return ' '      & Choose(bProcessing,'','')
     elsIf gData.Status = 'waiting_for_capture' then Return ' ' & Choose(bProcessing,'','')
     elsIf gData.Status = 'succeeded'           then Return Choose(bProcessing,'','') & ' '
     elsIf gData.Status = 'canceled'            then Return Choose(bProcessing,'','') & ' ͨ'.
  end
  
  Return ' ' 
  
!!!    1199 " "
YKassa::Calc1199   Procedure(Byte bTaxRate = 255,Bool bNoNDS = False) !,Byte
  Code
  If    bNoNDS   = True then Return 1   !      
  elsIf bTaxRate =   0  then Return 2
  elsIf bTaxRate =  10  then Return 3
  elsIf bTaxRate =  20  then Return 4
  elsIf bTaxRate = 110  then Return 5
  elsIf bTaxRate = 120  then Return 6
                        else Return 1.  !   
                       
!!!    2108 "  "
YKassa::Calc2108 Procedure(Long lOKEI,String sUnit)  !,Byte
lTag2108  Byte(YKASSA_IU_OTHER)
  Code
  If InList(lOKEI,796,642,812,778,868,728,881,625,736,872,870,839) 
                          lTag2108 = YKASSA_IU_PIECE             !  0. ,  ...

  elsIf lOKEI =  163 then lTag2108 = YKASSA_IU_GRAM              ! 10. 
  elsIf lOKEI =  166 then lTag2108 = YKASSA_IU_KILOGRAM          ! 11. 
  elsIf lOKEI =  168 then lTag2108 = YKASSA_IU_TON               ! 12. 
  elsIf lOKEI =    4 then lTag2108 = YKASSA_IU_CENTIMETER        ! 20. 
  elsIf lOKEI =    5 then lTag2108 = YKASSA_IU_SQUARE_DECIMETER  ! 21. 
  elsIf lOKEI =    6 then lTag2108 = YKASSA_IU_METER             ! 22. 
  elsIf lOKEI =   51 then lTag2108 = YKASSA_IU_SQUARE_CENTIMETER ! 30.  
  elsIf lOKEI =   53 then lTag2108 = YKASSA_IU_SQUARE_DECIMETER  ! 31.  
  elsIf lOKEI =   55 then lTag2108 = YKASSA_IU_SQUARE_METER      ! 32.  
  elsIf lOKEI =  111 then lTag2108 = YKASSA_IU_MILLILITER        ! 40. 
  elsIf lOKEI =  112 then lTag2108 = YKASSA_IU_LITER             ! 41. 
  elsIf lOKEI =  113 then lTag2108 = YKASSA_IU_CUBIC_METER       ! 42.  
  elsIf lOKEI =  245 then lTag2108 = YKASSA_IU_KILOWATT_HOUR     ! 50.  
  elsIf lOKEI =  233 then lTag2108 = YKASSA_IU_GKAL              ! 51. 
  elsIf lOKEI =  359 then lTag2108 = YKASSA_IU_DAY               ! 70.  ()
  elsIf lOKEI =  356 then lTag2108 = YKASSA_IU_HOUR              ! 71. 
  elsIf lOKEI =  355 then lTag2108 = YKASSA_IU_MINUTE            ! 72. 
  elsIf lOKEI =  354 then lTag2108 = YKASSA_IU_SECOND            ! 73. 
  elsIf lOKEI =  256 then lTag2108 = YKASSA_IU_KILOBYTE          ! 80. 
  elsIf lOKEI =  257 then lTag2108 = YKASSA_IU_MEGABYTE          ! 81. 
  elsIf lOKEI = 2553 then lTag2108 = YKASSA_IU_GIGABYTE          ! 82. 
  elsIf lOKEI = 2554 then lTag2108 = YKASSA_IU_TERABYTE          ! 83. 

  else  !  . 

     ! ---  

     Case Clip(Left(Lower(sUnit)))
     Of ''
        lTag2108 = YKASSA_IU_GRAM
     Of ''
        lTag2108 = YKASSA_IU_KILOGRAM
     Of ''
        lTag2108 = YKASSA_IU_TON
     Of ''
        lTag2108 = YKASSA_IU_CENTIMETER
     Of ''
        lTag2108 = YKASSA_IU_SQUARE_DECIMETER
     Of ''
        lTag2108 = YKASSA_IU_METER
     Of ' ' OrOf '. ' OrOf '..' OrOf '. .'
        lTag2108 = YKASSA_IU_SQUARE_CENTIMETER
     Of ' '  OrOf '. ' OrOf '..' OrOf '. .'
        lTag2108 = YKASSA_IU_SQUARE_DECIMETER
     Of ' '      OrOf '. '  OrOf '..'  OrOf '. .'  OrOf '. '
        lTag2108 = YKASSA_IU_SQUARE_METER
     Of ''
        lTag2108 = YKASSA_IU_MILLILITER
     Of ''
        lTag2108 = YKASSA_IU_LITER
     Of ' '      OrOf '. ' OrOf '..' OrOf '. .' OrOf '. '
        lTag2108 = YKASSA_IU_CUBIC_METER
     Of ' '         OrOf '..' OrOf '.'  OrOf '. .' OrOf '. '
        lTag2108 = YKASSA_IU_KILOWATT_HOUR
     Of ''          OrOf ''
        lTag2108 = YKASSA_IU_GKAL
     Of '' OrOf ''
        lTag2108 = YKASSA_IU_DAY
     Of '' OrOf 'kbyte'   OrOf 'kb'
        lTag2108 = YKASSA_IU_KILOBYTE
     Of '' OrOf 'mbyte'   OrOf 'mb'
        lTag2108 = YKASSA_IU_MEGABYTE
     Of '' OrOf 'gbyte'   OrOf 'gb'
        lTag2108 = YKASSA_IU_GIGABYTE
     Of '' OrOf 'tbyte'   OrOf 'tb'
        lTag2108 = YKASSA_IU_TERABYTE
     of '' OrOf '' OrOf ''  OrOf '' OrOf '' OrOf '' OrOf ''     OrOf '' OrOf '' OrOf '' |
                OrOf ''    OrOf '' OrOf ''    OrOf ''  OrOf ''   OrOf ''
        lTag2108 = YKASSA_IU_PIECE
     end

     ! --- 3 

     If lTag2108 = YKASSA_IU_OTHER

        Case Clip(Sub(Clip(Left(Lower(sUnit))),1,3))
        Of '2'
           lTag2108 = YKASSA_IU_SQUARE_CENTIMETER
        Of '2'
           lTag2108 = YKASSA_IU_SQUARE_DECIMETER
        Of '.'
           lTag2108 = YKASSA_IU_MILLILITER
        Of '.' OrOf ''
           lTag2108 = YKASSA_IU_LITER
        Of '' OrOf '.'
           lTag2108 = YKASSA_IU_DAY
        Of ''
           lTag2108 = YKASSA_IU_HOUR
        Of ''
           lTag2108 = YKASSA_IU_MINUTE
        Of ''
           lTag2108 = YKASSA_IU_SECOND
        end
     end

     ! --- 2 

     If lTag2108 = YKASSA_IU_OTHER

        Case Clip(Sub(Clip(Left(Lower(sUnit))),1,2))
        Of '' OrOf '' OrOf '' OrOf '' OrOf '' OrOf '' OrOf '' OrOf '' OrOf '' OrOf '' |
                OrOf '' OrOf '' OrOf '' OrOf '' OrOf '' OrOf ''
           lTag2108 = YKASSA_IU_PIECE
        of '' OrOf '.'
           lTag2108 = YKASSA_IU_GRAM
        Of ''
           lTag2108 = YKASSA_IU_KILOGRAM
        of '' OrOf '.'
           lTag2108 = YKASSA_IU_TON
        Of ''
           lTag2108 = YKASSA_IU_CENTIMETER
        Of ''
           lTag2108 = YKASSA_IU_SQUARE_DECIMETER
        Of '' OrOf '.' OrOf ''
           lTag2108 = YKASSA_IU_METER
        Of '2'
           lTag2108 = YKASSA_IU_SQUARE_METER
        Of ''
           lTag2108 = YKASSA_IU_MILLILITER
        Of '.'
           lTag2108 = YKASSA_IU_LITER
        Of '3'
           lTag2108 = YKASSA_IU_CUBIC_METER
        Of '' OrOf ''
           lTag2108 = YKASSA_IU_DAY
        Of '.'
           lTag2108 = YKASSA_IU_HOUR
        Of '.'
           lTag2108 = YKASSA_IU_SECOND
        Of '' OrOf 'kb'
           lTag2108 = YKASSA_IU_KILOBYTE
        Of '' OrOf 'mb'
           lTag2108 = YKASSA_IU_MEGABYTE
        Of '' OrOf 'gb'
           lTag2108 = YKASSA_IU_GIGABYTE
        Of '' OrOf 'tb'
           lTag2108 = YKASSA_IU_TERABYTE
        end
     end

     ! --- 1 

     If lTag2108 = YKASSA_IU_OTHER

        Case Clip(Sub(Clip(Left(Lower(sUnit))),1,1))
        Of ''
           lTag2108 = YKASSA_IU_GRAM
        Of ''
           lTag2108 = YKASSA_IU_TON
        Of ''
           lTag2108 = YKASSA_IU_METER
        Of ''
           lTag2108 = YKASSA_IU_LITER
        Of ''
           lTag2108 = YKASSA_IU_HOUR
        Of ''
           lTag2108 = YKASSA_IU_SECOND
        end
     end
  end

  Return lTag2108

!!!    2108 "  "
YKassa::Nick2108 Procedure(Long lOKEI,String sUnit)  !,String
bCode  Byte,Auto
  Code
  bCode = YKassa::Calc2108(lOKEI,sUnit)

  If    bCode =  0 then Return 'piece'              ! , 
  ElsIf bCode = 10 then Return 'gram'               ! 
  ElsIf bCode = 11 then Return 'kilogram'           !    
  ElsIf bCode = 12 then Return 'ton'                ! 
  ElsIf bCode = 20 then Return 'centimeter'         ! 
  ElsIf bCode = 21 then Return 'decimeter'          ! 
  ElsIf bCode = 22 then Return 'meter'              ! 
  ElsIf bCode = 30 then Return 'square_centimeter'  !  
  ElsIf bCode = 31 then Return 'square_decimeter'   !  
  ElsIf bCode = 32 then Return 'square_meter'       !  
  ElsIf bCode = 40 then Return 'milliliter'         ! 
  ElsIf bCode = 41 then Return 'liter'              ! 
  ElsIf bCode = 42 then Return 'cubic_meter'        !  
  ElsIf bCode = 50 then Return 'kilowatt_hour'      !  
  ElsIf bCode = 51 then Return 'gigacalorie'        ! 
  ElsIf bCode = 70 then Return 'day'                !  ()
  ElsIf bCode = 71 then Return 'hour'               ! 
  ElsIf bCode = 72 then Return 'minute'             ! 
  ElsIf bCode = 73 then Return 'second'             ! 
  ElsIf bCode = 80 then Return 'kilobyte'           ! 
  ElsIf bCode = 81 then Return 'megabyte'           ! 
  ElsIf bCode = 82 then Return 'gigabyte'           ! 
  ElsIf bCode = 83 then Return 'terabyte'.          ! 
  Return 'another'                                  !   

!!!      ITU-T E.164
YKassa::NormPhone Procedure(String sPhone) !,String
sRetVal CString(Size(sPhone) + 1)
  Code
  sRetVal = Clip(Left(DeFormat(sPhone)))
  If sRetVal <> '' and Sub(sRetVal,1,1) = '8'
     sRetVal = '7' & Clip(Sub(sRetVal,2,Len(sRetVal) - 1))
  end
  
  Return sRetVal
  
!!!    1212 "  "
YKassa::Nick1212 Procedure(Byte bObject,Bool bTestShop = False) !,String
  Code  
  If bTestShop and bObject > 13                   !      1.05
     Return 'another'                             
  else                                            ! *  !    
     Execute bObject
       Return 'commodity'                         ! 01 - 
       Return 'another'     ! 'excise'            ! * 02 -   ( )
       Return 'job'                               ! 03 - 
       Return 'service'                           ! 04 - 
       Return 'gambling_bet'                      ! 05 -   
       Return 'gambling_prize'                    ! 06 -   
       Return 'lottery'                           ! 07 -  
       Return 'lottery_prize'                     ! 08 -  
       Return 'intellectual_activity'             ! 09 -    
       Return 'payment'                           ! 10 - 
       Return 'agent_commission'                  ! 11 -  
       Return 'fine'                              ! 12 - 
       Return 'another'                           ! 13 -    (   1.05)
       Return 'property_right'                    ! 14 -  
       Return 'non_operating_gain'                ! 15 -  
       Return 'tax'                               ! 16 -    / : insurance_premium ?
       Return 'sales_tax'                         ! 17 -  
       Return 'resort_fee'                        ! 18 -  
       Return 'lien'                              ! 19 - 
       Return 'cost'                              ! 20 - 
       Return 'pension_insurance_without_payouts' ! 21 -    
       Return 'pension_insurance_with_payouts'    ! 22 -   
       Return 'health_insurance_without_payouts'  ! 23 -    
       Return 'health_insurance_with_payouts'     ! 24 -   
       Return 'health_insurance'                  ! 25 -   
       Return 'casino'                            ! 26 -  
       Return 'agent_withdrawals'                 ! 27 -   
       Return 'another'                           ! * 28 - 
       Return 'another'                           ! * 29 -   
       Return 'another'                           ! * 30 -  ,     ( )
       Return 'another'                           ! * 31 -  ,       ( ) 
       Return 'non_marked'                        ! 32 - ,    
       Return 'marked'                            ! 33 - ,   
     end
  end   
  Return 'another'                                ! 

!!!    1214 "  " ( Kassa   )
YKassa::Nick1214 Procedure(Byte bMethod = 0) !,String             
  Code
  If bMethod = 1 then Return 'full_prepayment'.
  Return 'full_payment'
  
!!!     
!YK::MyTest  Procedure()
!Root       &cJson
!Json       cJSONFactory
!  Code
  